@@ -1,7 +1,6 @@
 /* Fork a Unix child process, and set up to debug it, for GDB.
 
-   Copyright (C) 1990-1996, 1998-2001, 2004-2012 Free Software
-   Foundation, Inc.
+   Copyright (C) 1990-2014 Free Software Foundation, Inc.
 
    Contributed by Cygnus Support.
 
@@ -21,18 +20,18 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include "defs.h"
-#include "gdb_string.h"
+#include <string.h>
 #include "inferior.h"
 #include "terminal.h"
 #include "target.h"
 #include "gdb_wait.h"
 #include "gdb_vfork.h"
 #include "gdbcore.h"
-#include "terminal.h"
 #include "gdbthread.h"
 #include "command.h" /* for dont_repeat () */
 #include "gdbcmd.h"
 #include "solib.h"
+#include "filestuff.h"
 
 #include <signal.h>
 
@@ -150,11 +149,11 @@ fork_inferior (char *exec_file_arg, char *allargs, char **env,
   if (exec_file == 0)
     exec_file = get_exec_file (1);
 
-  /* STARTUP_WITH_SHELL is defined in inferior.h.  If 0,e we'll just
-    do a fork/exec, no shell, so don't bother figuring out what
-    shell.  */
+  /* 'startup_with_shell' is declared in inferior.h and bound to the
+     "set startup-with-shell" option.  If 0, we'll just do a
+     fork/exec, no shell, so don't bother figuring out what shell.  */
   shell_file = shell_file_arg;
-  if (STARTUP_WITH_SHELL)
+  if (startup_with_shell)
     {
       /* Figure out what shell to start up the user program under.  */
       if (shell_file == NULL)
@@ -272,9 +271,6 @@ fork_inferior (char *exec_file_arg, char *allargs, char **env,
       argv[3] = (char *) 0;
     }
 
-  /* On some systems an exec will fail if the executable is open.  */
-  close_exec_file ();
-
   /* Retain a copy of our environment variables, since the child will
      replace the value of environ and if we're vforked, we have to
      restore it.  */
@@ -317,6 +313,8 @@ fork_inferior (char *exec_file_arg, char *allargs, char **env,
 
   if (pid == 0)
     {
+      close_most_fds ();
+
       if (debug_fork)
 	sleep (debug_fork);
 
@@ -420,6 +418,12 @@ startup_inferior (int ntraps)
   int terminal_initted = 0;
   ptid_t resume_ptid;
 
+  if (startup_with_shell)
+    {
+      /* One trap extra for exec'ing the shell.  */
+      pending_execs++;
+    }
+
   if (target_supports_multi_process ())
     resume_ptid = pid_to_ptid (ptid_get_pid (inferior_ptid));
   else
@@ -434,7 +438,7 @@ startup_inferior (int ntraps)
 
   while (1)
     {
-      enum target_signal resume_signal = TARGET_SIGNAL_0;
+      enum gdb_signal resume_signal = GDB_SIGNAL_0;
       ptid_t event_ptid;
 
       struct target_waitstatus ws;
@@ -461,8 +465,8 @@ startup_inferior (int ntraps)
 	    target_terminal_ours ();
 	    target_mourn_inferior ();
 	    error (_("During startup program terminated with signal %s, %s."),
-		   target_signal_to_name (ws.value.sig),
-		   target_signal_to_string (ws.value.sig));
+		   gdb_signal_to_name (ws.value.sig),
+		   gdb_signal_to_string (ws.value.sig));
 	    return;
 
 	  case TARGET_WAITKIND_EXITED:
@@ -478,7 +482,7 @@ startup_inferior (int ntraps)
 	  case TARGET_WAITKIND_EXECD:
 	    /* Handle EXEC signals as if they were SIGTRAP signals.  */
 	    xfree (ws.value.execd_pathname);
-	    resume_signal = TARGET_SIGNAL_TRAP;
+	    resume_signal = GDB_SIGNAL_TRAP;
 	    switch_to_thread (event_ptid);
 	    break;
 
@@ -488,7 +492,7 @@ startup_inferior (int ntraps)
 	    break;
 	}
 
-      if (resume_signal != TARGET_SIGNAL_TRAP)
+      if (resume_signal != GDB_SIGNAL_TRAP)
 	{
 	  /* Let shell child handle its own signals in its own way.  */
 	  target_resume (resume_ptid, 0, resume_signal);
@@ -517,7 +521,7 @@ startup_inferior (int ntraps)
 	    break;
 
 	  /* Just make it go on.  */
-	  target_resume (resume_ptid, 0, TARGET_SIGNAL_0);
+	  target_resume (resume_ptid, 0, GDB_SIGNAL_0);
 	}
     }
 
@@ -534,6 +538,15 @@ unset_exec_wrapper_command (char *args, int from_tty)
   exec_wrapper = NULL;
 }
 
+static void
+show_startup_with_shell (struct ui_file *file, int from_tty,
+			 struct cmd_list_element *c, const char *value)
+{
+  fprintf_filtered (file,
+		    _("Use of shell to start subprocesses is %s.\n"),
+		    value);
+}
+
 /* Provide a prototype to silence -Wmissing-prototypes.  */
 extern initialize_file_ftype _initialize_fork_child;
 
@@ -551,4 +564,12 @@ Show the wrapper for running programs."), NULL,
   add_cmd ("exec-wrapper", class_run, unset_exec_wrapper_command,
            _("Disable use of an execution wrapper."),
            &unsetlist);
+
+  add_setshow_boolean_cmd ("startup-with-shell", class_support,
+			   &startup_with_shell, _("\
+Set use of shell to start subprocesses.  The default is on."), _("\
+Show use of shell to start subprocesses."), NULL,
+			   NULL,
+			   show_startup_with_shell,
+			   &setlist, &showlist);
 }
