@@ -1,5 +1,5 @@
-/* MI Command Set - breakpoint and watchpoint commands.
-   Copyright (C) 2000-2002, 2007-2012 Free Software Foundation, Inc.
+/* MI Command Set - file commands.
+   Copyright (C) 2000-2014 Free Software Foundation, Inc.
    Contributed by Cygnus Solutions (a Red Hat company).
 
    This file is part of GDB.
@@ -27,7 +27,7 @@
 #include "psymtab.h"
 
 /* Return to the client the absolute path and line number of the 
-   current file being executed. */
+   current file being executed.  */
 
 void
 mi_cmd_file_list_exec_source_file (char *command, char **argv, int argc)
@@ -38,31 +38,28 @@ mi_cmd_file_list_exec_source_file (char *command, char **argv, int argc)
   if (!mi_valid_noargs ("-file-list-exec-source-file", argc, argv))
     error (_("-file-list-exec-source-file: Usage: No args"));
 
-  /* Set the default file and line, also get them */
+  /* Set the default file and line, also get them.  */
   set_default_source_symtab_and_line ();
   st = get_current_source_symtab_and_line ();
 
-  /* We should always get a symtab. 
-     Apparently, filename does not need to be tested for NULL.
-     The documentation in symtab.h suggests it will always be correct */
+  /* We should always get a symtab.  Apparently, filename does not
+     need to be tested for NULL.  The documentation in symtab.h
+     suggests it will always be correct.  */
   if (!st.symtab)
     error (_("-file-list-exec-source-file: No symtab"));
 
-  /* Extract the fullname if it is not known yet */
-  symtab_to_fullname (st.symtab);
-
-  /* Print to the user the line, filename and fullname */
+  /* Print to the user the line, filename and fullname.  */
   ui_out_field_int (uiout, "line", st.line);
-  ui_out_field_string (uiout, "file", st.symtab->filename);
+  ui_out_field_string (uiout, "file",
+		       symtab_to_filename_for_display (st.symtab));
 
-  /* We may not be able to open the file (not available). */
-  if (st.symtab->fullname)
-  ui_out_field_string (uiout, "fullname", st.symtab->fullname);
+  ui_out_field_string (uiout, "fullname", symtab_to_fullname (st.symtab));
 
   ui_out_field_int (uiout, "macro-info", st.symtab->macro_table ? 1 : 0);
 }
 
 /* A callback for map_partial_symbol_filenames.  */
+
 static void
 print_partial_file_name (const char *filename, const char *fullname,
 			 void *ignore)
@@ -89,21 +86,16 @@ mi_cmd_file_list_exec_source_files (char *command, char **argv, int argc)
   if (!mi_valid_noargs ("-file-list-exec-source-files", argc, argv))
     error (_("-file-list-exec-source-files: Usage: No args"));
 
-  /* Print the table header */
+  /* Print the table header.  */
   ui_out_begin (uiout, ui_out_type_list, "files");
 
-  /* Look at all of the symtabs */
+  /* Look at all of the symtabs.  */
   ALL_SYMTABS (objfile, s)
   {
     ui_out_begin (uiout, ui_out_type_tuple, NULL);
 
-    ui_out_field_string (uiout, "file", s->filename);
-
-    /* Extract the fullname if it is not known yet */
-    symtab_to_fullname (s);
-
-    if (s->fullname)
-      ui_out_field_string (uiout, "fullname", s->fullname);
+    ui_out_field_string (uiout, "file", symtab_to_filename_for_display (s));
+    ui_out_field_string (uiout, "fullname", symtab_to_fullname (s));
 
     ui_out_end (uiout, ui_out_type_tuple);
   }
